"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("@nativescript/core/utils/types");
var Https = require("./https.common");
var file_system_1 = require("@nativescript/core/file-system");
var image_source_1 = require("@nativescript/core/image-source");
var cache;
function setCache(options) {
    if (options) {
        cache = NSURLCache.alloc().initWithMemoryCapacityDiskCapacityDirectoryURL(options.memorySize, options.diskSize, NSURL.URLWithString(options.diskLocation));
    }
    else {
        cache = null;
    }
    NSURLCache.sharedURLCache = cache;
}
exports.setCache = setCache;
function clearCache() {
    NSURLCache.sharedURLCache.removeAllCachedResponses();
}
exports.clearCache = clearCache;
var policies = {
    def: AFSecurityPolicy.defaultPolicy(),
    secured: false,
};
policies.def.allowInvalidCertificates = true;
policies.def.validatesDomainName = false;
function enableSSLPinning(options) {
    if (!policies.secure) {
        policies.secure = AFSecurityPolicy.policyWithPinningMode(1);
        policies.secure.allowInvalidCertificates = types_1.isDefined(options.allowInvalidCertificates)
            ? options.allowInvalidCertificates
            : false;
        policies.secure.validatesDomainName = types_1.isDefined(options.validatesDomainName)
            ? options.validatesDomainName
            : true;
        var data = NSData.dataWithContentsOfFile(options.certificate);
        policies.secure.pinnedCertificates = NSSet.setWithObject(data);
    }
    policies.secured = true;
}
exports.enableSSLPinning = enableSSLPinning;
function disableSSLPinning() {
    policies.secured = false;
}
exports.disableSSLPinning = disableSSLPinning;
function nativeToObj(data) {
    var content;
    if (data instanceof NSDictionary) {
        content = {};
        data.enumerateKeysAndObjectsUsingBlock(function (key, value, stop) {
            content[key] = nativeToObj(value);
        });
        return content;
    }
    else if (data instanceof NSArray) {
        content = [];
        data.enumerateObjectsUsingBlock(function (value, index, stop) {
            content[index] = nativeToObj(value);
        });
        return content;
    }
    else if (data instanceof NSData) {
        return NSString.alloc()
            .initWithDataEncoding(data, NSASCIIStringEncoding)
            .toString();
    }
    else {
        return data;
    }
}
function getData(data) {
    var content;
    if (data && data.class) {
        if (data.enumerateKeysAndObjectsUsingBlock ||
            data instanceof NSArray) {
            var serial = NSJSONSerialization.dataWithJSONObjectOptionsError(data, 1);
            content = NSString.alloc()
                .initWithDataEncoding(serial, NSUTF8StringEncoding)
                .toString();
        }
        else if (data instanceof NSData) {
            content = NSString.alloc()
                .initWithDataEncoding(data, NSASCIIStringEncoding)
                .toString();
        }
        else {
            content = data;
        }
        try {
            content = JSON.parse(content);
        }
        catch (ignore) { }
    }
    else if (typeof data === "object") {
        content = JSON.stringify(data);
    }
    else {
        content = data;
    }
    return content;
}
var HttpsResponse = (function () {
    function HttpsResponse(data, url) {
        this.data = data;
        this.url = url;
    }
    HttpsResponse.prototype.toArrayBufferAsync = function () {
        throw new Error("Method not implemented.");
    };
    HttpsResponse.prototype.toArrayBuffer = function () {
        if (this.arrayBuffer) {
            return this.arrayBuffer;
        }
        if (this.data instanceof NSData) {
            this.arrayBuffer = interop.bufferFromData(this.data);
        }
        else {
            this.arrayBuffer = interop.bufferFromData(NSKeyedArchiver.archivedDataWithRootObject(this.data));
        }
        return this.arrayBuffer;
    };
    HttpsResponse.prototype.toString = function (encoding) {
        if (this.stringResponse) {
            return this.stringResponse;
        }
        if (this.jsonResponse) {
            this.stringResponse = JSON.stringify(this.jsonResponse);
            return this.stringResponse;
        }
        if (typeof this.data === "string") {
            this.stringResponse = this.data;
            return this.data;
        }
        else {
            var data = nativeToObj(this.data);
            if (typeof data === "string") {
                this.stringResponse = data;
            }
            else {
                this.jsonResponse = data;
                this.stringResponse = JSON.stringify(data);
            }
            return this.stringResponse;
        }
    };
    HttpsResponse.prototype.toStringAsync = function (encoding) {
        return Promise.resolve(this.toString(encoding));
    };
    HttpsResponse.prototype.toJSON = function (encoding) {
        if (this.jsonResponse) {
            return this.jsonResponse;
        }
        if (this.stringResponse) {
            this.jsonResponse = Https.parseJSON(this.stringResponse);
            return this.jsonResponse;
        }
        var data = nativeToObj(this.data);
        if (typeof data === "object") {
            this.jsonResponse = data;
            return data;
        }
        try {
            this.stringResponse = data;
            this.jsonResponse = Https.parseJSON(data);
            return this.jsonResponse;
        }
        catch (err) {
            console.error("HttpsResponse.toJSON", err);
            return null;
        }
    };
    HttpsResponse.prototype.toJSONAsync = function () {
        return Promise.resolve(this.toJSON());
    };
    HttpsResponse.prototype.toImage = function () {
        var _this = this;
        if (this.imageSource) {
            return Promise.resolve(this.imageSource);
        }
        return new Promise(function (resolve, reject) {
            UIImage.tns_decodeImageWithDataCompletion(_this.data, function (image) {
                if (image) {
                    resolve(new image_source_1.ImageSource(image));
                }
                else {
                    reject(new Error("Response content may not be converted to an Image"));
                }
            });
        }).then(function (r) {
            _this.imageSource = r;
            return r;
        });
    };
    HttpsResponse.prototype.toFile = function (destinationFilePath) {
        var _this = this;
        if (this.file) {
            return Promise.resolve(this.file);
        }
        return new Promise(function (resolve, reject) {
            if (!destinationFilePath) {
                destinationFilePath = Https.getFilenameFromUrl(_this.url);
            }
            if (_this.data instanceof NSData) {
                var file = file_system_1.File.fromPath(destinationFilePath);
                var result = _this.data.writeToFileAtomically(destinationFilePath, true);
                if (resolve) {
                    resolve(file);
                }
                else {
                    reject(new Error("Cannot save file with path: " + destinationFilePath + "."));
                }
            }
            else {
                reject(new Error("Cannot save file with path: " + destinationFilePath + "."));
            }
        }).then(function (f) {
            _this.file = f;
            return f;
        });
    };
    return HttpsResponse;
}());
function AFFailure(resolve, reject, task, error, useLegacy, url) {
    if (error.code === -999) {
        return reject(new Error(error.localizedDescription));
    }
    var getHeaders = function () { return ({}); };
    var sendi = {
        task: task,
        reason: error.localizedDescription,
        get headers() {
            return getHeaders();
        },
    };
    var response = error.userInfo.valueForKey(AFNetworkingOperationFailingURLResponseErrorKey);
    if (!types_1.isNullOrUndefined(response)) {
        sendi.statusCode = response.statusCode;
        getHeaders = function () {
            var dict = response.allHeaderFields;
            if (dict) {
                var headers_1 = {};
                dict.enumerateKeysAndObjectsUsingBlock(function (k, v) { return (headers_1[k] = v); });
                return headers_1;
            }
            return null;
        };
    }
    var data = error.userInfo.valueForKey(AFNetworkingOperationFailingURLResponseDataErrorKey);
    var parsedData = getData(data);
    var failingURL = error.userInfo.objectForKey("NSErrorFailingURLKey");
    if (useLegacy) {
        var failure = {
            description: error.description,
            reason: error.localizedDescription,
            url: failingURL ? failingURL.description : url,
        };
        if (policies.secured === true) {
            failure.description =
                "nativescript-https > Invalid SSL certificate! " +
                    error.description;
        }
        sendi.failure = failure;
        sendi.content = new HttpsResponse(data, url);
        resolve(sendi);
    }
    else {
        var content = {
            body: parsedData,
            description: error.description,
            reason: error.localizedDescription,
            url: failingURL ? failingURL.description : url,
        };
        if (policies.secured === true) {
            content.description =
                "nativescript-https > Invalid SSL certificate! " +
                    content.description;
        }
        sendi.content = content;
        resolve(sendi);
    }
}
function bodyToNative(cont) {
    var dict;
    if (Array.isArray(cont)) {
        dict = NSArray.arrayWithArray(cont.map(function (item) { return bodyToNative(item); }));
    }
    else if (types_1.isObject(cont)) {
        dict = NSMutableDictionary.new();
        Object.keys(cont).forEach(function (key) {
            return dict.setValueForKey(bodyToNative(cont[key]), key);
        });
    }
    else {
        dict = cont;
    }
    return dict;
}
function createRequest(opts) {
    var manager = AFHTTPSessionManager.alloc().initWithBaseURL(NSURL.URLWithString(opts.url));
    var type = opts.headers && opts.headers["Content-Type"]
        ? opts.headers["Content-Type"]
        : "application/json";
    if (type.startsWith("application/json")) {
        manager.requestSerializer = AFJSONRequestSerializer.serializer();
        manager.responseSerializer = AFJSONResponseSerializer.serializerWithReadingOptions(4);
    }
    else {
        manager.requestSerializer = AFHTTPRequestSerializer.serializer();
        manager.responseSerializer = AFHTTPResponseSerializer.serializer();
    }
    manager.requestSerializer.allowsCellularAccess = true;
    manager.requestSerializer.HTTPShouldHandleCookies = true;
    manager.securityPolicy =
        policies.secured === true ? policies.secure : policies.def;
    if (opts.cachePolicy) {
        switch (opts.cachePolicy) {
            case "noCache":
                manager.setDataTaskWillCacheResponseBlock(function (session, task, cacheResponse) {
                    return null;
                });
                break;
            case "onlyCache":
                manager.requestSerializer.cachePolicy =
                    3;
                break;
            case "ignoreCache":
                manager.requestSerializer.cachePolicy =
                    1;
                break;
        }
    }
    var heads = opts.headers;
    var headers = null;
    if (heads) {
        headers = NSMutableDictionary.dictionary();
        Object.keys(heads).forEach(function (key) { return headers.setValueForKey(heads[key], key); });
    }
    var dict = null;
    if (opts.body) {
        dict = bodyToNative(opts.body);
    }
    else if (opts.content) {
        dict = opts.content;
    }
    manager.requestSerializer.timeoutInterval = opts.timeout
        ? opts.timeout
        : 10;
    var useLegacy = types_1.isDefined(opts.useLegacy) ? opts.useLegacy : false;
    var progress = opts.onProgress
        ? function (progress) {
            opts.onProgress(progress.completedUnitCount, progress.totalUnitCount);
        }
        : null;
    var task;
    return {
        get nativeRequest() {
            return task;
        },
        cancel: function () { return task && task.cancel(); },
        run: function (resolve, reject) {
            var success = function (task, data) {
                var content = useLegacy
                    ? new HttpsResponse(data, opts.url)
                    : getData(data);
                var getHeaders = function () { return ({}); };
                var sendi = {
                    content: content,
                    get headers() {
                        return getHeaders();
                    },
                };
                var response = task.response;
                if (!types_1.isNullOrUndefined(response)) {
                    sendi.statusCode = response.statusCode;
                    getHeaders = function () {
                        var dict = response.allHeaderFields;
                        if (dict) {
                            var headers_2 = {};
                            dict.enumerateKeysAndObjectsUsingBlock(function (k, v) { return (headers_2[k] = v); });
                            return headers_2;
                        }
                        return null;
                    };
                }
                resolve(sendi);
            };
            var failure = function (task, error) {
                AFFailure(resolve, reject, task, error, useLegacy, opts.url);
            };
            if (type.startsWith("multipart/form-data")) {
                switch (opts.method) {
                    case "POST":
                        headers.removeObjectForKey("Content-Type");
                        task = manager.POSTParametersHeadersConstructingBodyWithBlockProgressSuccessFailure(opts.url, null, headers, function (formData) {
                            opts.body.forEach(function (param) {
                                if (param.fileName &&
                                    param.contentType) {
                                    if (param.data instanceof NSURL) {
                                        formData.appendPartWithFileURLNameFileNameMimeTypeError(param.data, param.parameterName, param.fileName, param.contentType);
                                    }
                                    else {
                                        formData.appendPartWithFileDataNameFileNameMimeType(param.data, param.parameterName, param.fileName, param.contentType);
                                    }
                                }
                                else {
                                    formData.appendPartWithFormDataName(NSString.stringWithString(param.data).dataUsingEncoding(NSUTF8StringEncoding), param.parameterName);
                                }
                            });
                        }, progress, success, failure);
                        break;
                    default:
                        throw new Error("method_not_supported_multipart");
                }
            }
            else {
                switch (opts.method) {
                    case "GET":
                        task = manager.GETParametersHeadersProgressSuccessFailure(opts.url, dict, headers, progress, success, failure);
                        break;
                    case "POST":
                        task = manager.POSTParametersHeadersProgressSuccessFailure(opts.url, dict, headers, progress, success, failure);
                        break;
                    case "PUT":
                        task = manager.PUTParametersHeadersSuccessFailure(opts.url, dict, headers, success, failure);
                        break;
                    case "DELETE":
                        task = manager.DELETEParametersHeadersSuccessFailure(opts.url, dict, headers, success, failure);
                        break;
                    case "PATCH":
                        task = manager.PATCHParametersHeadersSuccessFailure(opts.url, dict, headers, success, failure);
                        break;
                    case "HEAD":
                        task = manager.HEADParametersHeadersSuccessFailure(opts.url, dict, headers, success, failure);
                        break;
                    default:
                        throw new Error("method_not_supported_multipart");
                }
            }
        },
    };
}
exports.createRequest = createRequest;
function request(opts) {
    return new Promise(function (resolve, reject) {
        try {
            createRequest(opts).run(resolve, reject);
        }
        catch (error) {
            reject(error);
        }
    });
}
exports.request = request;
//# sourceMappingURL=https.ios.js.map